@@ -5,13 +5,15 @@ from http.server import BaseHTTPRequestHandler
 import os
 import socket
 from socketserver import ThreadingMixIn
+import ssl
 import sys
 import threading
-from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple
+from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple, Union
 from urllib.error import HTTPError
 from urllib.parse import parse_qs, quote_plus, urlparse
 from urllib.request import (
-    build_opener, HTTPHandler, HTTPRedirectHandler, Request,
+    BaseHandler, build_opener, HTTPHandler, HTTPRedirectHandler, HTTPSHandler,
+    Request,
 )
 from wsgiref.simple_server import make_server, WSGIRequestHandler, WSGIServer
 
@@ -36,7 +38,6 @@ __all__ = (
 
 CONTENT_TYPE_LATEST = 'text/plain; version=0.0.4; charset=utf-8'
 """Content type of the latest text format"""
-PYTHON376_OR_NEWER = sys.version_info > (3, 7, 5)
 
 
 class _PrometheusRedirectHandler(HTTPRedirectHandler):
@@ -97,10 +98,10 @@ class _PrometheusRedirectHandler(HTTPRedirectHandler):
 def _bake_output(registry, accept_header, accept_encoding_header, params, disable_compression):
     """Bake output for metrics output."""
     # Choose the correct plain text format of the output.
-    formatter, content_type = choose_formatter(accept_header)
+    encoder, content_type = choose_encoder(accept_header)
     if 'name[]' in params:
         registry = registry.restricted_registry(params['name[]'])
-    output = formatter(registry)
+    output = encoder(registry)
     headers = [('Content-Type', content_type)]
     # If gzip encoding required, gzip the output.
     if not disable_compression and gzip_accepted(accept_encoding_header):
@@ -237,7 +238,7 @@ def generate_latest(registry: CollectorRegistry = REGISTRY) -> bytes:
     return ''.join(output).encode('utf-8')
 
 
-def choose_formatter(accept_header: str) -> Tuple[Callable[[CollectorRegistry], bytes], str]:
+def choose_encoder(accept_header: str) -> Tuple[Callable[[CollectorRegistry], bytes], str]:
     accept_header = accept_header or ''
     for accepted in accept_header.split(','):
         if accepted.split(';')[0].strip() == 'application/openmetrics-text':
@@ -314,7 +315,7 @@ def _make_handler(
         timeout: Optional[float],
         headers: Sequence[Tuple[str, str]],
         data: bytes,
-        base_handler: type,
+        base_handler: Union[BaseHandler, type],
 ) -> Callable[[], None]:
     def handle() -> None:
         request = Request(url, data=data)
@@ -368,8 +369,8 @@ def basic_auth_handler(
         timeout: Optional[float],
         headers: List[Tuple[str, str]],
         data: bytes,
-        username: str = None,
-        password: str = None,
+        username: Optional[str] = None,
+        password: Optional[str] = None,
 ) -> Callable[[], None]:
     """Handler that implements HTTP/HTTPS connections with Basic Auth.
 
@@ -389,6 +390,40 @@ def basic_auth_handler(
     return handle
 
 
+def tls_auth_handler(
+        url: str,
+        method: str,
+        timeout: Optional[float],
+        headers: List[Tuple[str, str]],
+        data: bytes,
+        certfile: str,
+        keyfile: str,
+        cafile: Optional[str] = None,
+        protocol: int = ssl.PROTOCOL_TLS_CLIENT,
+        insecure_skip_verify: bool = False,
+) -> Callable[[], None]:
+    """Handler that implements an HTTPS connection with TLS Auth.
+
+    The default protocol (ssl.PROTOCOL_TLS_CLIENT) will also enable
+    ssl.CERT_REQUIRED and SSLContext.check_hostname by default. This can be
+    disabled by setting insecure_skip_verify to True.
+    
+    Both this handler and the TLS feature on pushgateay are experimental."""
+    context = ssl.SSLContext(protocol=protocol)
+    if cafile is not None:
+        context.load_verify_locations(cafile)
+    else:
+        context.load_default_certs()
+
+    if insecure_skip_verify:
+        context.check_hostname = False
+        context.verify_mode = ssl.CERT_NONE
+
+    context.load_cert_chain(certfile=certfile, keyfile=keyfile)
+    handler = HTTPSHandler(context=context)
+    return _make_handler(url, method, timeout, headers, data, handler)
+
+
 def push_to_gateway(
         gateway: str,
         job: str,
@@ -509,10 +544,7 @@ def _use_gateway(
 ) -> None:
     gateway_url = urlparse(gateway)
     # See https://bugs.python.org/issue27657 for details on urlparse in py>=3.7.6.
-    if not gateway_url.scheme or (
-            PYTHON376_OR_NEWER
-            and gateway_url.scheme not in ['http', 'https']
-    ):
+    if not gateway_url.scheme or gateway_url.scheme not in ['http', 'https']:
         gateway = f'http://{gateway}'
 
     gateway = gateway.rstrip('/')
